/* Includes */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

/* Constantes */
#define MAXENLACE 2	/* N de enlaces de salida de un estado permitido por el programa */
#define MAXAUTOMA  20	/* N mximo de estados de un autmata permitido por el programa */
#define MAXPATRON 100   /* N mximo permitido por el programa para el largo de un patrn */

/* variables globales */
struct alfabeto
	{
	unsigned short enlace[MAXENLACE]; /* Enlace a otra letra del alfabeto */
	char letra[MAXENLACE];		  /* Letra del alfabeto */
	};

struct alfabeto automata[MAXAUTOMA];	  /* Automata que representa a la gramtica vlida */
unsigned long Npatrones;		  /* N de patrones */
unsigned short enlace_c,                  /* Estado inicial */
	       enlace_f,                  /* Estado final */
	       Maxtamanio;		  /* Tamao mximo de un patrn */

/*--------------------------- abrir_fichero --------------------------------*/
FILE *abrir_fichero(char *nombref)
/*
FINALIDAD: Abrir un fichero.
*/
{
FILE *fich_ent;	/* Fichero de entrada */

if ((fich_ent = fopen(nombref,"r")) == NULL)
	{
	clrscr();
	printf("ERROR: El fichero %s no de puede abrir\n", nombref);
	exit(1);
	}
return(fich_ent);
}

/*--------------------------- crear_fichero --------------------------------*/
FILE *crear_fichero(char *nombref)
/*
FINALIDAD: Crear un fichero.
*/
{
FILE *fich_sal;	/* Fichero de entrada */

if ((fich_sal = fopen(nombref,"w")) == NULL)
	{
	clrscr();
	printf("ERROR: El fichero %s no de puede crear\n", nombref);
	exit(1);
	}
return(fich_sal);
}

/*--------------------------- lee_entrada ----------------------------------*/

void lee_entrada(char *nombref)
/*
FINALIDAD: Leer desde un fichero el autmata que describe la gramtica.
*/
{
unsigned short Nestados,  /* N de estados */
	       i,
	       enlace;    /* Prximo estado */
FILE *fich_ent;		  /* Fichero de entrada */

fich_ent = abrir_fichero(nombref);
fscanf(fich_ent, "%ld",&Npatrones);	/* Lee el n de patrones */
fscanf(fich_ent,"%d",&Maxtamanio); /* Tamao mximo de los patrones */
printf("\n\n Nmero de patrones       = %ld", Npatrones);
printf("\n Tamao mximo del patrn = %d\n", Maxtamanio);
fscanf(fich_ent,"%d",&Nestados);  /* N de estados */
fscanf(fich_ent,"%d",&enlace_c);  /* Estado inicial */
fscanf(fich_ent,"%d %c",&automata[enlace_c].enlace[0], &automata[enlace_c].letra[0]);
for (i = 0; i < Nestados - 3; ++i)
	{
	fscanf(fich_ent,"%d",&enlace);  /* Estados intermedios */
	fscanf(fich_ent,"%d %d %c %c", &automata[enlace].enlace[0], &automata[enlace].enlace[1],
	       &automata[enlace].letra[0], &automata[enlace].letra[1]);
	}
fscanf(fich_ent,"%d",&enlace_f);  /* Estado final */
fscanf(fich_ent,"%d %c", &automata[enlace_f].enlace[0], &automata[enlace_f].letra[0]);
close(fich_ent);
}

/*--------------------------- aletarorio -----------------------------------*/
unsigned short aleatorio()
/*
FINALIDAD: Retornar un 0..MAXENLACE aletatoriamente
*/
{
return(random(MAXENLACE));
}

/*--------------------------- genera_gramatica -----------------------------*/

void genera_gramatica(char *nombref)
/*
FINALIDAD: Generar la gramtica vlida para el autmata de entrada
*/
{
unsigned long  Npatrones_gen;  /* Patrones generados hasta el momento */
unsigned short enlace;         /* Prximo enlace */
char patron[MAXPATRON];        /* Patrn */
unsigned short tamanio,	       /* Tamao actual de patrn */
	       aleat;	       /* N aleatorio (0  1) */
FILE *fich_sal;		       /* Fichero de salida */

randomize(); /* Inicializa los nmeros aleatorios */
fich_sal = crear_fichero(nombref);
Npatrones_gen = 0;
tamanio = 0;
strcpy(patron,"\0");
while (Npatrones_gen < Npatrones)
	{
	patron[tamanio++] =  automata[enlace_c].letra[0];  /* Letra de comienzo */
	enlace = automata[enlace_c].enlace[0];
	while ((enlace != enlace_f) && (tamanio <= Maxtamanio))
		{
		aleat = aleatorio();                        /* N aleatorio (0  1) */
		patron[tamanio++] = automata[enlace].letra[aleat]; /* Copia la letra en el patrn */
		enlace = automata[enlace].enlace[aleat];       /* Prximo enlace con probabilidad equitativa */
		}
	if (tamanio <= Maxtamanio)	/* Se gener un patrn dentro del tamao permitido */
		{
		patron[tamanio++] = automata[enlace_f].letra[0]; /* Copia la letra final */
		patron[tamanio] = '\0';
		fprintf(fich_sal,"%s\n", patron);
		++Npatrones_gen;
		printf(" Patrones generados: %6ld\r", Npatrones_gen);
		}
	strcpy(patron,"\0");
	tamanio = 0;
	}
close(fich_sal);
}
/*--------------------------- MAIN -----------------------------------------*/
void main(int argc, char *argv[])
{
clrscr();
if (argc != 3)
	{
	printf("\n\n AYUDA: GEN_GRAM fichero_ejecucin fichero_patrones", argv[2]);
	printf("\n          donde: fichero_ejecucin -> Es el fichero de entrada con la");
	printf("                                                 definicin de la gramtica.");
	printf("\n                 fichero_patrones  -> Es el fichero de salida con patrones");
	printf("                                            que pertenecen a dicha gramtica.\n\n");
	exit(1);
	}
printf("\n************************** Generar Patrones (gramtica) **********************");
lee_entrada(argv[1]);
genera_gramatica(argv[2]);
printf("\n\n  El fichero %s que contiene los patrones se gener correctamente !! \n\n", argv[2]);
}